# Elecraft K3 Rig Control Python Utilities
# Copyright (C) 2006, 2007, 2008 Leigh L. Klotz, Jr <Leigh@WA5ZNU.org>
# Licensed under Academic Free License 3.0 (See LICENSE)

import string, serial, time, os
from threading import Lock
from lxml import etree

class K3Rig:
    configFiles = [os.path.expanduser("~/.fldigi/rig.xml"), os.path.expanduser("~/.k3rig/rig.xml")]

    def __init__(self):
        self.levels = None
        self.lock = Lock()
        self.config_date = None
        self.read_config_file()

    def read_config_file(self):
        for configFile in self.configFiles:
            if  os.path.exists(configFile):
                t = os.path.getmtime(configFile)
                if self.config_date == t:
                    return
                else:
                    self.config_date = t
                    config = etree.parse(configFile).getroot()
                    assert config.tag == "RIGDEF"
                    self.DEVICE=config.findtext("PORT/DEVICE")
                    self.BAUDRATE=config.findtext("PORT/BAUD")
                    return

    def firmware_levels(self):
        return self.levels

    # RV[MDAVF]; -> RV[MDAVF]*nn.nn;
    # Returns None if not installed or cannot be read
    def verq(self, x):
        cmd = "RV%c;" % (x)
        self.ser.write(cmd)
        result = self.ser.read(9);
        if (len(result) <> 9):
            return None
        if result[0:3] != cmd[0:3]:
            return None
        ver = result[3:8]
        if ver[2] != '.':
            return None
        if ver == '99.99':
            ver = None
        else:
            return ver[0:2]+""+ver[3:5]

    def update(self):
        #print "Rig update"
        self.lock.acquire()
        try:
            l = FirmwareLevels()
            self.ser = serial.Serial(self.DEVICE,baudrate=self.BAUDRATE,rtscts=0, timeout=1)
            self.ser.write("AI0;K30;")
            self.ser.flushInput()
            self.ser.flushOutput()
            l.mcu = self.verq('M')
            l.dsp1 = self.verq('D')
            l.dsp2 = self.verq('A')
            l.dvr = self.verq('V');
            l.fp = self.verq('F')
            self.ser.flushInput()
            self.ser.close()
            if l.mcu and l.dsp1:
                self.levels = l
                return True
            else:
                return False
        finally:
            self.lock.release()

class FirmwareLevels:
    def __init__(self):
        self.mcu = None
        self.dsp1 = None
        self.dsp2 = None
        self.dvr = None
        self.fp = None  

    def isNewer(self, other):
        if other:
            if other.mcu > self.mcu:
                return True
            if other.dsp1 > self.dsp1:
                return True
            if other.fp > self.fp:
                return True
            if other.dvr > self.dvr:
                return True
        return False

    def isOlder(self, other):
        if other:
            if other.mcu < self.mcu:
                return True
            if other.dsp1 < self.dsp1:
                return True
            if other.fp < self.fp:
                return True
            if other.dvr < self.dvr:
                return True
        return False

    def version_string(self):
        buf = ""
        if self.mcu:
            buf += "uC: %s" % (self.mcu)
        if self.dsp1:
            buf += " DSP1: %s" % (self.dsp1)
        if self.dsp2:
            buf += " DSP2: %s" % (self.dsp2)
        if self.fp:
            buf += " FP: %s" % (self.fp)
        if self.dvr:
            buf += " DVR: %s" % (self.dvr)
        return buf
